
#ifndef VTKPVVTKEXTENSIONSFILTERSRENDERING_EXPORT_H
#define VTKPVVTKEXTENSIONSFILTERSRENDERING_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSFILTERSRENDERING_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSFILTERSRENDERING_EXPORT
#  define VTKPVVTKEXTENSIONSFILTERSRENDERING_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSFILTERSRENDERING_EXPORT
#    ifdef VTKExtensionsFiltersRendering_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSFILTERSRENDERING_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSFILTERSRENDERING_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSFILTERSRENDERING_NO_EXPORT
#    define VTKPVVTKEXTENSIONSFILTERSRENDERING_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSFILTERSRENDERING_DEPRECATED
#  define VTKPVVTKEXTENSIONSFILTERSRENDERING_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSFILTERSRENDERING_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSFILTERSRENDERING_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSFILTERSRENDERING_EXPORT VTKPVVTKEXTENSIONSFILTERSRENDERING_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSFILTERSRENDERING_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSFILTERSRENDERING_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSFILTERSRENDERING_NO_EXPORT VTKPVVTKEXTENSIONSFILTERSRENDERING_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSFILTERSRENDERING_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSFILTERSRENDERING_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSFILTERSRENDERING_EXPORT_H */
