
#ifndef VTKPVVTKEXTENSIONSIOEXODUS_EXPORT_H
#define VTKPVVTKEXTENSIONSIOEXODUS_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSIOEXODUS_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSIOEXODUS_EXPORT
#  define VTKPVVTKEXTENSIONSIOEXODUS_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSIOEXODUS_EXPORT
#    ifdef VTKExtensionsIOExodus_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSIOEXODUS_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSIOEXODUS_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSIOEXODUS_NO_EXPORT
#    define VTKPVVTKEXTENSIONSIOEXODUS_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSIOEXODUS_DEPRECATED
#  define VTKPVVTKEXTENSIONSIOEXODUS_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSIOEXODUS_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSIOEXODUS_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSIOEXODUS_EXPORT VTKPVVTKEXTENSIONSIOEXODUS_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSIOEXODUS_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSIOEXODUS_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSIOEXODUS_NO_EXPORT VTKPVVTKEXTENSIONSIOEXODUS_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSIOEXODUS_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSIOEXODUS_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSIOEXODUS_EXPORT_H */
