/*=========================================================================

  Program:   ParaView
  Module:    vtk_protobuf.h

  Copyright (c) Kitware, Inc.
  All rights reserved.
  See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#ifndef vtk_protobuf_h
#define vtk_protobuf_h

/* Use the protobuf library configured for ParaView.  */
#define VTK_MODULE_USE_EXTERNAL_ParaView_protobuf 1

#if VTK_MODULE_USE_EXTERNAL_ParaView_protobuf
# include <google/protobuf/message.h>
#else
# include <vtkprotobuf/src/google/protobuf/message.h>
#endif

#endif
