
#ifndef VTKSURFACELICREPRESENTATIONS_EXPORT_H
#define VTKSURFACELICREPRESENTATIONS_EXPORT_H

#ifdef VTKSURFACELICREPRESENTATIONS_STATIC_DEFINE
#  define VTKSURFACELICREPRESENTATIONS_EXPORT
#  define VTKSURFACELICREPRESENTATIONS_NO_EXPORT
#else
#  ifndef VTKSURFACELICREPRESENTATIONS_EXPORT
#    ifdef vtkSurfaceLICRepresentations_EXPORTS
        /* We are building this library */
#      define VTKSURFACELICREPRESENTATIONS_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKSURFACELICREPRESENTATIONS_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKSURFACELICREPRESENTATIONS_NO_EXPORT
#    define VTKSURFACELICREPRESENTATIONS_NO_EXPORT 
#  endif
#endif

#ifndef VTKSURFACELICREPRESENTATIONS_DEPRECATED
#  define VTKSURFACELICREPRESENTATIONS_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKSURFACELICREPRESENTATIONS_DEPRECATED_EXPORT
#  define VTKSURFACELICREPRESENTATIONS_DEPRECATED_EXPORT VTKSURFACELICREPRESENTATIONS_EXPORT VTKSURFACELICREPRESENTATIONS_DEPRECATED
#endif

#ifndef VTKSURFACELICREPRESENTATIONS_DEPRECATED_NO_EXPORT
#  define VTKSURFACELICREPRESENTATIONS_DEPRECATED_NO_EXPORT VTKSURFACELICREPRESENTATIONS_NO_EXPORT VTKSURFACELICREPRESENTATIONS_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKSURFACELICREPRESENTATIONS_NO_DEPRECATED
#    define VTKSURFACELICREPRESENTATIONS_NO_DEPRECATED
#  endif
#endif

#endif /* VTKSURFACELICREPRESENTATIONS_EXPORT_H */
