
#ifndef VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_EXPORT_H
#define VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_EXPORT
#  define VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_EXPORT
#    ifdef VTKExtensionsFiltersMaterialInterface_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_NO_EXPORT
#    define VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_DEPRECATED
#  define VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_EXPORT VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_NO_EXPORT VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSFILTERSMATERIALINTERFACE_EXPORT_H */
