



python -c "from paraview.simple import VisItNek5000Reader"
IF %ERRORLEVEL% NEQ 0 exit /B 1
python -c "from paraview.simple import Xdmf3ReaderS"
IF %ERRORLEVEL% NEQ 0 exit /B 1
pvpython --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
